function [th,Plls]=linfit_ol(D)
% Function for fitting a linear segment with possible outliers.
%
% function [th,Plls]=linfit_ol(D)
%
% D    Processed data, are assumed to fit model:
%
%        d(1) = th d(2:end) + e
% where th is a vector of unknown linear coefficients
%       e  is a Gaussian-distributed disturbance with
%          zero-mean and unknown variance
%
% output:
%   th     estimate of linear coefficients
%   Plls   probability of each point being

% Initial phase
% ==============

if 1
    SC = max(D(:,1))-min(D(:,1));
    D = D/SC;
end

% prior
dim = size(D,2);
V0 = zeros(dim);
nu0 = 0;
ndat = size(D,1);

llUni = -log(max(D(:,1))-min(D(:,1)))+log(1);

% likelihood of data
Plls = zeros(1,ndat);
% probably better initialization
PisC = 0.1;
Plls = sin((0.5:(ndat-0.5))/ndat*pi) + PisC;
Plls = Plls/(max(Plls)+0.01);
llP0 = log(1-Plls);
llP1 = log(Plls);

converged = 0;
old_Plls = 0*Plls;
while (converged<1)
    V = V0;
    nu = nu0;
    for j = 1:ndat
        rgr = D(j,:);
        V = V + rgr'*rgr * Plls(j);
        nu = nu + Plls(j);
    end
    % test probability of each point
    LLo = gwll(V,nu)+(ndat-nu)*llUni;
    lls = zeros(1,ndat);
    for j =1:ndat
        rgr = D(j,:);
        % likelihood of removal of the ith record
        lls0(j) = gwll(V-rgr'*rgr*Plls(j),nu-Plls(j))+...
            Plls(j)*llUni+llP0(j); % removal
        lls1(j) = gwll(V+rgr'*rgr*(1-Plls(j)),nu+(1-Plls(j)))-...
            (1-Plls(j))*llUni+llP1(j); % addition
    end
    Plls = exp(lls1)./(exp(lls1)+exp(lls0));
    if norm(Plls-old_Plls)<0.0001
        converged = 1;
    else
        old_Plls = Plls;
    end
end


th = inv(V(2:end,2:end))*V(2:end,1);

function ll=gwll(V,nu);
Vaa = V(2:end,2:end);
iVaa = inv(Vaa);
Lam = V(1,1) - V(1,2:end)*iVaa*V(2:end,1);
ll = gammaln(0.5*nu) -0.5*nu*log(Lam) ...
    - 0.5*1*log(det(Vaa)) + 1/2*nu*log(2) ...
    +1/2*(2-nu)*log(pi);
if ~isreal(ll)
   ll = -inf;
end
